@@ -33,6 +33,8 @@
 #include <gst/gst.h>
 #include <bonobo/bonobo-ui-util.h>
 
+#include <profiles/gnome-media-profiles.h>
+
 #include "gnome-recorder.h"
 #include "gsr-window.h"
 
@@ -51,6 +53,7 @@ typedef struct _GSRWindowPipeline {
 struct _GSRWindowPrivate {
 	GtkWidget *main_vbox, *ev;
 	GtkWidget *scale;
+	GtkWidget *profile;
 	GtkWidget *rate, *time_sec, *format, *channels;
 	GtkWidget *name, *length;
 
@@ -62,6 +65,7 @@ struct _GSRWindowPrivate {
 	/* Pipelines */
 	GSRWindowPipeline *play, *record;
 	char *filename, *record_filename;
+        char *extension;
 	char *working_file; /* Working file: Operations only occur on the
 			       working file. The result of that operation then
 			       becomes the new working file. */
@@ -78,8 +82,11 @@ struct _GSRWindowPrivate {
 
 	guint32 tick_id; /* tick_callback timeout ID */
 	guint32 record_id; /* record idle callback timeout ID */
+	guint32 gstenc_id; /* encode idle callback iterator ID */
 };
 
+void error_handler (GObject *object, GstObject *orig, gchar *error, GtkWidget *window);
+
 static BonoboWindowClass *parent_class = NULL;
 
 static void
@@ -323,60 +330,48 @@ file_new (BonoboUIComponent *uic,
 	gsr_open_window (NULL);
 }
 
-struct _FileOpenData {
-	GtkWidget *filesel;
-	GSRWindow *window;
-};
-
 static void
-file_sel_response (GtkDialog *filesel,
-		   int response_id,
-		   struct _FileOpenData *fd)
+file_chooser_open_response (GtkDialog *file_chooser,
+			    int response_id,
+			    GSRWindow *window)
 {
 	GConfClient *client;
-	const char *name;
-	char *dirname, *d;
+	char *name;
+	char *dirname;
 
 	switch (response_id) {
 	case GTK_RESPONSE_OK:
-		name = gtk_file_selection_get_filename (GTK_FILE_SELECTION (filesel));
+		name = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_chooser));
 
-		d = g_path_get_dirname (name);
-		/* Add a trailing / so that the file selector 
-		   will cd into it */
-		dirname = g_strdup_printf ("%s/", d);
+		if (name != NULL) {
+			dirname = g_path_get_dirname (name);
 
-		client = gconf_client_get_default ();
-		gconf_client_set_string (client,
-					 "/apps/gnome-sound-recorder/system-state/open-file-directory",
-					 dirname, NULL);
-		g_object_unref (G_OBJECT (client));
-		g_free (dirname);
-		g_free (d);
+			client = gconf_client_get_default ();
+			gconf_client_set_string (client,
+					 	"/apps/gnome-sound-recorder/system-state/open-file-directory",
+					 	dirname, NULL);
+			g_object_unref (G_OBJECT (client));
+			g_free (dirname);
 
-		if (fd->window->priv->has_file == TRUE) {
-			/* Just open a new window with the file */
-			
-			gsr_open_window (name);
-			
-			gtk_widget_destroy (GTK_WIDGET (filesel));
-			g_free (fd);
-			return;
-		} else {
-			/* Set the file in this window */
-			g_object_set (G_OBJECT (fd->window),
-				      "location", name,
-				      NULL);
+			if (window->priv->has_file == TRUE) {
+				/* Just open a new window with the file */
+				gsr_open_window (name);
+			} else {
+				/* Set the file in this window */
+				g_object_set (G_OBJECT (window),
+					      "location", name,
+					      NULL);
+			}
+
+			g_free (name);
 		}
 
 		break;
-
 	default:
 		break;
 	}
 
-	gtk_widget_destroy (GTK_WIDGET (filesel));
-	g_free (fd);
+	gtk_widget_destroy (GTK_WIDGET (file_chooser));
 }
 
 static void
@@ -384,29 +379,30 @@ file_open (BonoboUIComponent *uic,
 	   GSRWindow *window,
 	   const char *path)
 {
-	struct _FileOpenData *fd;
+	GtkWidget *file_chooser;
 	GConfClient *client;
 	char *directory;
 
-	fd = g_new (struct _FileOpenData, 1);
-	fd->window = window;
-	fd->filesel = gtk_file_selection_new (_("Open a file"));
-	gtk_window_set_transient_for (GTK_WINDOW (fd->filesel), GTK_WINDOW (window));
+	file_chooser = gtk_file_chooser_dialog_new (_("Open a file"),
+						    GTK_WINDOW (window),
+						    GTK_FILE_CHOOSER_ACTION_OPEN,
+						    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+						    GTK_STOCK_OPEN, GTK_RESPONSE_OK,
+						    NULL);
 
 	client = gconf_client_get_default ();
 	directory = gconf_client_get_string (client,
 					     "/apps/gnome-sound-recorder/system-state/open-file-directory",
 					     NULL);
 	if (directory != NULL && *directory != 0) {
-		gtk_file_selection_set_filename (GTK_FILE_SELECTION (fd->filesel),
-						 directory);
+		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (file_chooser), directory);
 	}
 	g_object_unref (G_OBJECT (client));
 	g_free (directory);
 
-	g_signal_connect (G_OBJECT (fd->filesel), "response",
-			  G_CALLBACK (file_sel_response), fd);
-	gtk_widget_show (fd->filesel);
+	g_signal_connect (G_OBJECT (file_chooser), "response",
+			  G_CALLBACK (file_chooser_open_response), window);
+	gtk_widget_show (file_chooser);
 }
 
 enum mimetype {
@@ -437,6 +433,7 @@ get_encoder_for_mimetype (enum mimetype mime)
 struct _eos_data {
 	GSRWindow *window;
 	char *location;
+	GstElement *pipeline;
 };
 
 static gboolean
@@ -444,6 +441,10 @@ eos_done (struct _eos_data *ed)
 {
 	GSRWindow *window = ed->window;
 
+ 	gst_element_set_state (ed->pipeline, GST_STATE_NULL);
+	g_source_remove (window->priv->gstenc_id);
+	window->priv->gstenc_id = 0;
+
 	g_object_set (G_OBJECT (window),
 		      "location", ed->location,
 		      NULL);
@@ -470,7 +471,6 @@ eos_done (struct _eos_data *ed)
 	bonobo_ui_component_set_status (window->priv->ui_component,
 					_("Ready"), NULL);
 
-	g_free (ed->location);
 	g_free (ed);
 
 	return FALSE;
@@ -485,18 +485,196 @@ save_sink_eos (GstElement *element,
 
 	ed = g_new (struct _eos_data, 1);
 	ed->window = window;
+	ed->pipeline = GST_ELEMENT (gst_element_get_parent (element));
 	g_object_get (G_OBJECT (element),
 		      "location", &ed->location,
 		      NULL);
 
-	g_idle_add ((GSourceFunc) eos_done, ed);
- 	gst_element_set_state (GST_ELEMENT (gst_element_get_parent (element)),
-			       GST_STATE_NULL);
+	/*g_idle_add ((GSourceFunc) eos_done, ed);*/
+	eos_done (ed);
+}
+
+#if 0
+static gboolean
+cb_iterate (GstBin  *bin,
+	    gpointer data)
+{
+	GSRWindow *window = data;
+	const GList *kids = bin->children;
+	GstElement *src = NULL, *sink = NULL;
+
+	for (; kids; kids = kids->next) {
+		if (!(strcmp (gst_object_get_name (GST_OBJECT (kids->data)), "src")))
+			src = kids->data;
+		else if (!(strcmp (gst_object_get_name (GST_OBJECT (kids->data)), "sink")))
+			sink = kids->data;
+	}
+
+	if (src && sink) {
+		gint64 pos, tot, enc;
+		GstFormat fmt = GST_FORMAT_BYTES;
+
+		gst_element_query (src, GST_QUERY_POSITION,
+				   &fmt, &pos);
+		gst_element_query (src, GST_QUERY_TOTAL,
+				   &fmt, &tot);
+		gst_element_query (sink, GST_QUERY_POSITION,
+				   &fmt, &enc);
+
+		g_print ("Iterate: %lld/%lld -> %lld\n", pos, tot, enc);
+	} else
+		g_print ("Iterate ?\n");
+
+	/* we don't do anything here */
+	return FALSE;
+}
+#endif
+
+static void
+cb_error (GstElement *parent,
+	  GstElement *cause,
+	  gchar      *reason,
+	  gpointer    data)
+{
+	GSRWindow *window = data;
+	struct _eos_data *ed;
+	const GList *kids = GST_BIN (parent)->children;
+	GstElement *sink;
+
+	while (kids && !(strcmp (gst_object_get_name (GST_OBJECT (kids->data)), "sink")))
+		kids = kids->next;
+	g_assert (kids);
+	sink = kids->data;
+
+	error_handler (G_OBJECT (parent), GST_OBJECT (cause), reason, data);
+
+	ed = g_new (struct _eos_data, 1);
+	ed->window = window;
+	ed->pipeline = parent;
+	g_object_get (G_OBJECT (sink),
+		      "location", &ed->location,
+		      NULL);
+
+	/*g_idle_add ((GSourceFunc) eos_done, ed);*/
+	eos_done (ed);
+}
+
+static GtkWidget*
+gsr_button_new_with_stock_image (const gchar* text, const gchar* stock_id)
+{
+    GtkWidget *button;
+    GtkStockItem item;
+    GtkWidget *label;
+    GtkWidget *image;
+    GtkWidget *hbox;
+    GtkWidget *align;
+
+    button = gtk_button_new ();
+
+    if (GTK_BIN (button)->child)
+            gtk_container_remove (GTK_CONTAINER (button),
+                                  GTK_BIN (button)->child);
+
+    if (gtk_stock_lookup (stock_id, &item))
+    {
+        label = gtk_label_new_with_mnemonic (text);
+
+        gtk_label_set_mnemonic_widget (GTK_LABEL (label), GTK_WIDGET (button));
+
+        image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_BUTTON);
+        hbox = gtk_hbox_new (FALSE, 2);
+
+        align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+
+        gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+        gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+
+        gtk_container_add (GTK_CONTAINER (button), align);
+        gtk_container_add (GTK_CONTAINER (align), hbox);
+        gtk_widget_show_all (align);
+
+        return button;
+    }
+    label = gtk_label_new_with_mnemonic (text);
+    gtk_label_set_mnemonic_widget (GTK_LABEL (label), GTK_WIDGET (button));
+
+    gtk_misc_set_alignment (GTK_MISC (label), 0.5, 0.5);
+
+    gtk_widget_show (label);
+    gtk_container_add (GTK_CONTAINER (button), label);
+
+    return button;
+}
+
+GtkWidget*
+gsr_dialog_add_button (GtkDialog *dialog, const gchar* text, const gchar* stock_id,
+                       gint response_id)
+{
+    GtkWidget *button;
+
+    g_return_val_if_fail (GTK_IS_DIALOG (dialog), NULL);
+    g_return_val_if_fail (text != NULL, NULL);
+    g_return_val_if_fail (stock_id != NULL, NULL);
+
+    button = gsr_button_new_with_stock_image (text, stock_id);
+    g_return_val_if_fail (button != NULL, NULL);
+
+    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+
+    gtk_widget_show (button);
+
+    gtk_dialog_add_action_widget (dialog, button, response_id);
+
+    return button;
+}
+
+static gboolean
+replace_dialog (GtkWindow *parent, const gchar *message, const gchar *file_name)
+{
+    GtkWidget *msgbox;
+    gint ret; 
+
+    g_return_val_if_fail (file_name != NULL, FALSE); 
+    msgbox = gtk_message_dialog_new (parent,
+                                     GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                                     GTK_MESSAGE_QUESTION,
+                                     GTK_BUTTONS_NONE,
+                                     message,
+                                     file_name);
+
+    /* Add Cancel button */
+    gtk_dialog_add_button (GTK_DIALOG (msgbox),
+                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
+
+    /* Add Replace button */
+    gsr_dialog_add_button (GTK_DIALOG (msgbox),
+                           _("_Replace"), GTK_STOCK_REFRESH, GTK_RESPONSE_YES);
+
+    gtk_dialog_set_default_response (GTK_DIALOG (msgbox), GTK_RESPONSE_CANCEL);
+
+    gtk_window_set_resizable (GTK_WINDOW (msgbox), FALSE);
+
+    ret = gtk_dialog_run (GTK_DIALOG (msgbox));
+
+    gtk_widget_destroy (msgbox);
+
+    return (ret == GTK_RESPONSE_YES);
+}
+
+static gboolean
+replace_existing_file (GtkWindow *parent, const gchar* file_name)
+{
+    return replace_dialog (parent,
+                           _("A file named \"%s\" already exists.\n"
+                             "Do you want to replace it with the "
+                             "one you are saving?"),
+                           file_name);
 }
 
 static void
 do_save_file (GSRWindow *window,
-	      const char *name)
+	      const char *name,
+          gboolean saveas)
 {
 	const char *ext;
 	char *status_text, *basename;
@@ -504,37 +682,47 @@ do_save_file (GSRWindow *window,
 	GstElement *pipeline, *src, *encoder, *spider, *sink;
 	GdkCursor *cursor;
 
-	ext = strrchr (name, '.');
-	if (ext == NULL || *ext == 0) {
-		mime = MIME_TYPE_WAV;
-	} else {
-		if (strcmp ("wav", ext + 1) == 0) {
-			mime = MIME_TYPE_WAV;
-		} else if (strcmp ("mp3", ext + 1) == 0) {
-			mime = MIME_TYPE_MP3;
-		} else if (strcmp ("ogg", ext + 1) == 0) {
-			mime = MIME_TYPE_OGG;
-		} else if (strcmp ("flac", ext + 1) == 0) {
-			mime = MIME_TYPE_FLAC;
-		}
+	if (saveas && g_file_test (name, G_FILE_TEST_EXISTS)) {
+		if (!replace_existing_file (GTK_WINDOW (window), name))
+                	return;
 	}
 
-	pipeline = gst_thread_new ("save-pipeline");
+	ext = strrchr (name, '.') + 1;
+	if (strcmp ("wav", ext) == 0) {
+		mime = MIME_TYPE_WAV;
+	} else if (strcmp ("mp3", ext) == 0) {
+		mime = MIME_TYPE_MP3;
+	} else if (strcmp ("ogg", ext) == 0) {
+		mime = MIME_TYPE_OGG;
+	} else if (strcmp ("flac", ext) == 0) {
+		mime = MIME_TYPE_FLAC;
+	} else {
+		error_handler (NULL, NULL,
+			       _("Unknown extension, doing nothing"),
+			       GTK_WIDGET (window));
+		return;
+	}
+
+	pipeline = gst_pipeline_new ("save-pipeline");
 	src = gst_element_factory_make ("filesrc", "src");
 	spider = gst_element_factory_make ("spider", "spider");
 	encoder = get_encoder_for_mimetype (mime);
 	sink = gst_element_factory_make ("filesink", "sink");
+	/* make this a cute error dialog */
+	if (!pipeline || !src || !spider || !encoder || !sink) {
+		error_handler (NULL, NULL,
+			       _("Could not build pipeline"),
+			       GTK_WIDGET (window));
+	}
 
-	gst_bin_add (GST_BIN (pipeline), src);
-	gst_bin_add (GST_BIN (pipeline), spider);
-	gst_bin_add (GST_BIN (pipeline), encoder);
-	gst_bin_add (GST_BIN (pipeline), sink);
+	gst_bin_add_many (GST_BIN (pipeline),
+			  src, spider, encoder, sink, NULL);
+	gst_element_link_many (src, spider, encoder, sink, NULL);
 
-	gst_element_connect (src, spider);
-	gst_element_connect (spider, encoder);
-	gst_element_connect (encoder, sink);
 	g_signal_connect (G_OBJECT (sink), "eos",
 			  G_CALLBACK (save_sink_eos), window);
+	g_signal_connect (G_OBJECT (pipeline), "error",
+			  G_CALLBACK (cb_error), window);
 
 	g_object_set (G_OBJECT (src), 
 		      "location", window->priv->working_file,
@@ -571,84 +759,83 @@ do_save_file (GSRWindow *window,
 	g_free (status_text);
 
 	gtk_widget_set_sensitive (window->priv->scale, FALSE);
-	
 	gst_element_set_state (pipeline, GST_STATE_PLAYING);
+	window->priv->gstenc_id = g_idle_add ((GSourceFunc) gst_bin_iterate, pipeline);
 }
 
 static void
-file_sel_save_response (GtkDialog *filesel,
-			int response_id,
-			struct _FileOpenData *fd)
+file_chooser_save_response (GtkDialog *file_chooser,
+			    int response_id,
+			    GSRWindow *window)
 {
 	GConfClient *client;
-	const char *name;
-	char *dirname, *d;
+	char *name;
+	char *dirname;
 
 	switch (response_id) {
 	case GTK_RESPONSE_OK:
-		name = gtk_file_selection_get_filename (GTK_FILE_SELECTION (filesel));
+		name = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_chooser));
 
-		d = g_path_get_dirname (name);
-		/* Add a trailing / so that the file selector 
-		   will cd into it */
-		dirname = g_strdup_printf ("%s/", d);
+		if (name != NULL) {
+			dirname = g_path_get_dirname (name);
 
-		client = gconf_client_get_default ();
-		gconf_client_set_string (client,
-					 "/apps/gnome-sound-recorder/system-state/save-file-directory",
-					 dirname, NULL);
-		g_object_unref (G_OBJECT (client));
-		g_free (dirname);
-		g_free (d);
+			client = gconf_client_get_default ();
+			gconf_client_set_string (client,
+					 	"/apps/gnome-sound-recorder/system-state/save-file-directory",
+					 	dirname, NULL);
+			g_object_unref (G_OBJECT (client));
+			g_free (dirname);
+
+			do_save_file (window, name, TRUE /* Is save as */);
+			g_free (name);
+		}
 
-		do_save_file (fd->window, name);
 		break;
-
 	default:
 		break;
 	}
 
-	gtk_widget_destroy (GTK_WIDGET (filesel));
-	g_free (fd);
+	gtk_widget_destroy (GTK_WIDGET (file_chooser));
 }
 
 static void
 file_save_as (BonoboUIComponent *uic,
-	   GSRWindow *window,
-	   const char *path)
+	      GSRWindow *window,
+	      const char *path)
 {
-	struct _FileOpenData *fd;
+	GtkWidget *file_chooser;
 	GConfClient *client;
 	char *directory;
 
-	fd = g_new (struct _FileOpenData, 1);
-	fd->window = window;
-	fd->filesel = gtk_file_selection_new (_("Save file as"));
-	gtk_window_set_transient_for (GTK_WINDOW (fd->filesel), GTK_WINDOW (window));
+	file_chooser = gtk_file_chooser_dialog_new (_("Save file as"),
+						    GTK_WINDOW (window),
+						    GTK_FILE_CHOOSER_ACTION_SAVE,
+						    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+						    GTK_STOCK_SAVE, GTK_RESPONSE_OK,
+						    NULL);
 
 	client = gconf_client_get_default ();
 	directory = gconf_client_get_string (client,
 					     "/apps/gnome-sound-recorder/system-state/save-file-directory",
 					     NULL);
 	if (directory != NULL && *directory != 0) {
-		gtk_file_selection_set_filename (GTK_FILE_SELECTION (fd->filesel),
-						 directory);
+		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (file_chooser), directory);
 	}
 	g_object_unref (G_OBJECT (client));
 	g_free (directory);
-	
+
 	if (window->priv->filename != NULL) {
 		char *basename;
 
 		basename = g_path_get_basename (window->priv->filename);
-		gtk_file_selection_set_filename (GTK_FILE_SELECTION (fd->filesel),
-						 basename);
+		gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (file_chooser),
+						   basename);
 		g_free (basename);
 	}
 
-	g_signal_connect (G_OBJECT (fd->filesel), "response",
-			  G_CALLBACK (file_sel_save_response), fd);
-	gtk_widget_show (fd->filesel);
+	g_signal_connect (G_OBJECT (file_chooser), "response",
+			  G_CALLBACK (file_chooser_save_response), window);
+	gtk_widget_show (file_chooser);
 }
 
 static void
@@ -660,7 +847,7 @@ file_save (BonoboUIComponent *uic,
 	    strncmp (window->priv->filename, "Untitled", 8) == 0) {
 		file_save_as (uic, window, NULL);
 	} else {
-		do_save_file (window, window->priv->filename);
+		do_save_file (window, window->priv->filename, FALSE /* Not Save as */);
 	}
 }
 
@@ -1384,6 +1571,46 @@ error_handler (GObject *object, GstObject *orig, gchar *error, GtkWidget *window
 	gtk_widget_destroy (dialog);
 }
 
+/* helper function to change the UI when file extension changes */
+static void
+set_extension (GSRWindow *window)
+{
+	gchar *short_name, *filename;
+        GMAudioProfile *profile;
+
+        g_object_get (window, "location", &filename, NULL);
+	short_name = g_path_get_basename (filename);
+        profile = gm_audio_profile_choose_get_active (window->priv->profile);
+        g_free (window->priv->extension);
+        window->priv->extension = g_strdup (gm_audio_profile_get_extension (profile));
+	if (filename != NULL) {
+		char *title;
+
+		title = g_strdup_printf (_("%s.%s - Sound Recorder"), short_name, window->priv->extension);
+		gtk_window_set_title (GTK_WINDOW (window), title);
+		g_free (title);
+	} else {
+		gtk_window_set_title (GTK_WINDOW (window), _("Sound Recorder"));
+	}
+
+}
+
+/* callback for when the recording profile has been changed */
+static void
+profile_changed (GObject *object, GSRWindow *window)
+{
+  GMAudioProfile *profile;
+  gchar *partial_pipeline;
+  gchar *extension;
+
+  g_return_if_fail (GTK_IS_COMBO_BOX (object));
+  profile = gm_audio_profile_choose_get_active (GTK_WIDGET (object));
+  partial_pipeline = g_strdup (gm_audio_profile_get_pipeline (profile));
+
+  set_extension (window);
+
+  
+}
 
 static GSRWindowPipeline *
 make_play_pipeline (GSRWindow *window)
@@ -1406,12 +1633,9 @@ make_play_pipeline (GSRWindow *window)
  	spider = gst_element_factory_make ("spider", "spider");
 	pipeline->sink = gst_element_factory_make ("osssink", "sink");
 
-	gst_bin_add (GST_BIN (pipeline->pipeline), pipeline->src);
- 	gst_bin_add (GST_BIN (pipeline->pipeline), spider); 
-	gst_bin_add (GST_BIN (pipeline->pipeline), pipeline->sink);
-
-	gst_element_connect (pipeline->src, spider);
-	gst_element_connect (spider, pipeline->sink);
+	gst_bin_add_many (GST_BIN (pipeline->pipeline),
+			  pipeline->src, spider, pipeline->sink, NULL);
+	gst_element_link_many (pipeline->src, spider, pipeline->sink, NULL);
 
 	return pipeline;
 }
@@ -1516,12 +1740,9 @@ make_record_pipeline (GSRWindow *window)
 	}
 	pipeline->sink = gst_element_factory_make ("filesink", "sink");
 
-	gst_bin_add (GST_BIN (pipeline->pipeline), pipeline->src);
-	gst_bin_add (GST_BIN (pipeline->pipeline), encoder);
-	gst_bin_add (GST_BIN (pipeline->pipeline), pipeline->sink);
-
-	gst_element_connect (pipeline->src, encoder);
-	gst_element_connect (encoder, pipeline->sink);
+	gst_bin_add_many (GST_BIN (pipeline->pipeline),
+			  pipeline->src, encoder, pipeline->sink, NULL);
+	gst_element_link_many (pipeline->src, encoder, pipeline->sink, NULL);
 
 	return pipeline;
 }
@@ -1570,6 +1791,7 @@ init (GSRWindow *window)
 	priv->len_secs = 0;
 	priv->get_length_attempts = 16;
 	priv->dirty = FALSE;
+	priv->gstenc_id = 0;
 
 	return;
 }
@@ -1618,9 +1840,11 @@ gsr_window_new (const char *filename)
 	struct stat buf;
 	char *short_name;
 
+        /* filename has been changed to be without extension */
 	window = g_object_new (GSR_WINDOW_TYPE, 
 			       "location", filename,
 			       NULL);
+        /* FIXME: check extension too */
 	window->priv->filename = g_strdup (filename);
 	if (stat (filename, &buf) == 0) {
 		window->priv->has_file = TRUE;
@@ -1628,17 +1852,6 @@ gsr_window_new (const char *filename)
 		window->priv->has_file = FALSE;
 	}
 
-	short_name = g_path_get_basename (filename);
-	if (filename != NULL) {
-		char *title;
-
-		title = g_strdup_printf (_("%s - Sound Recorder"), short_name);
-		gtk_window_set_title (GTK_WINDOW (window), title);
-		g_free (title);
-	} else {
-		gtk_window_set_title (GTK_WINDOW (window), _("Sound Recorder"));
-	}
-
 	window->priv->record_filename = g_strdup_printf ("%s/gsr-record-%s-%d.XXXXXX",
 							 g_get_tmp_dir(), filename, getpid ());
 	window->priv->record_fd = mkstemp (window->priv->record_filename);
@@ -1679,9 +1892,22 @@ gsr_window_new (const char *filename)
 	gtk_widget_show (window->priv->scale);
 	gtk_box_pack_start (GTK_BOX (window->priv->main_vbox), window->priv->scale, TRUE, TRUE, 0);
 
-	vbox = gtk_vbox_new (FALSE, 6);
+	vbox = gtk_vbox_new (FALSE, 7);
 	gtk_box_pack_start (GTK_BOX (window->priv->main_vbox), vbox, TRUE, TRUE, 0);
 
+	hbox = gtk_hbox_new (FALSE, 2);
+	gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
+
+	label = gtk_label_new (_("Record as"));
+	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+        window->priv->profile = gm_audio_profile_choose_new ();
+	gtk_box_pack_start (GTK_BOX (hbox), window->priv->profile,
+                            FALSE, FALSE, 0);
+        gtk_widget_show (window->priv->profile);
+        g_signal_connect (G_OBJECT (window->priv->profile), "changed",
+                          G_CALLBACK (profile_changed), window);
+
 	label = gtk_label_new (_("File information"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 
@@ -1703,6 +1929,7 @@ gsr_window_new (const char *filename)
 			  0, 1, 0, 1,
 			  GTK_FILL, GTK_FILL, 0, 0);
 
+        short_name = g_path_get_basename (filename);
 	window->priv->name = gtk_label_new (short_name ? short_name : _("<none>"));
 	gtk_label_set_selectable (GTK_LABEL (window->priv->name), TRUE);
 	gtk_label_set_line_wrap (GTK_LABEL (window->priv->name), GTK_WRAP_WORD);
@@ -1726,6 +1953,8 @@ gsr_window_new (const char *filename)
 			  GTK_FILL, GTK_FILL,
 			  0, 0);
 			       
+        set_extension (window);
+
 	gtk_widget_show_all (window->priv->main_vbox);
 	return GTK_WIDGET (window);
 }
